@@ -19,15 +19,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /home/projects/pgsql/cvsroot/pgsql/src/bin/pg_dump/pg_backup_custom.c,v 1.10 2001/04/01 05:42:51 pjw Exp $
- *
- * Modifications - 28-Jun-2000 - pjw@rhyme.com.au
- *
- *	Initial version.
- *
- * Modifications - 04-Jan-2001 - pjw@rhyme.com.au
- *
- *	  - Check results of IO routines more carefully.
+ *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_custom.c,v 1.23 2002/10/25 01:33:17 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -35,7 +27,6 @@
 #include "pg_backup.h"
 #include "pg_backup_archiver.h"
 
-#include <stdlib.h>
 #include <errno.h>
 
 /*--------
@@ -45,12 +36,12 @@
 
 static void _ArchiveEntry(ArchiveHandle *AH, TocEntry *te);
 static void _StartData(ArchiveHandle *AH, TocEntry *te);
-static int	_WriteData(ArchiveHandle *AH, const void *data, int dLen);
+static size_t _WriteData(ArchiveHandle *AH, const void *data, size_t dLen);
 static void _EndData(ArchiveHandle *AH, TocEntry *te);
 static int	_WriteByte(ArchiveHandle *AH, const int i);
 static int	_ReadByte(ArchiveHandle *);
-static int	_WriteBuf(ArchiveHandle *AH, const void *buf, int len);
-static int	_ReadBuf(ArchiveHandle *AH, void *buf, int len);
+static size_t _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len);
+static size_t _ReadBuf(ArchiveHandle *AH, void *buf, size_t len);
 static void _CloseArchive(ArchiveHandle *AH);
 static void _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt);
 static void _WriteExtraToc(ArchiveHandle *AH, TocEntry *te);
@@ -80,16 +71,16 @@ typedef struct
 	z_streamp	zp;
 	char	   *zlibOut;
 	char	   *zlibIn;
-	int			inSize;
+	size_t		inSize;
 	int			hasSeek;
-	int			filePos;
-	int			dataStart;
+	off_t		filePos;
+	off_t		dataStart;
 } lclContext;
 
 typedef struct
 {
-	int			dataPos;
-	int			dataLen;
+	int			dataState;
+	off_t		dataPos;
 } lclTocEntry;
 
 
@@ -100,10 +91,10 @@ typedef struct
 static void _readBlockHeader(ArchiveHandle *AH, int *type, int *id);
 static void _StartDataCompressor(ArchiveHandle *AH, TocEntry *te);
 static void _EndDataCompressor(ArchiveHandle *AH, TocEntry *te);
-static int	_getFilePos(ArchiveHandle *AH, lclContext *ctx);
+static off_t _getFilePos(ArchiveHandle *AH, lclContext *ctx);
 static int	_DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush);
 
-static char *progname = "Archiver(custom)";
+static char *modulename = gettext_noop("custom archiver");
 
 
 
@@ -147,12 +138,18 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
 	 */
 	ctx = (lclContext *) malloc(sizeof(lclContext));
 	if (ctx == NULL)
-		die_horribly(AH, "%s: Unable to allocate archive context", progname);
+		die_horribly(AH, modulename, "out of memory\n");
 	AH->formatData = (void *) ctx;
 
 	ctx->zp = (z_streamp) malloc(sizeof(z_stream));
 	if (ctx->zp == NULL)
-		die_horribly(AH, "%s: unable to allocate zlib stream archive context", progname);
+		die_horribly(AH, modulename, "out of memory\n");
+
+	/* Initialize LO buffering */
+	AH->lo_buf_size = LOBBUFSIZE;
+	AH->lo_buf = (void *) malloc(LOBBUFSIZE);
+	if (AH->lo_buf == NULL)
+		die_horribly(AH, modulename, "out of memory\n");
 
 	/*
 	 * zlibOutSize is the buffer size we tell zlib it can output to.  We
@@ -167,36 +164,33 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
 	ctx->filePos = 0;
 
 	if (ctx->zlibOut == NULL || ctx->zlibIn == NULL)
-		die_horribly(AH, "%s: unable to allocate buffers in archive context", progname);
+		die_horribly(AH, modulename, "out of memory\n");
 
 	/*
 	 * Now open the file
 	 */
 	if (AH->mode == archModeWrite)
 	{
-
 		if (AH->fSpec && strcmp(AH->fSpec, "") != 0)
 			AH->FH = fopen(AH->fSpec, PG_BINARY_W);
 		else
 			AH->FH = stdout;
 
 		if (!AH->FH)
-			die_horribly(AH, "%s: unable to open archive file %s", progname, AH->fSpec);
-
-		ctx->hasSeek = (fseek(AH->FH, 0, SEEK_CUR) == 0);
+			die_horribly(AH, modulename, "could not open archive file %s: %s\n", AH->fSpec, strerror(errno));
 
+		ctx->hasSeek = checkSeek(AH->FH);
 	}
 	else
 	{
-
 		if (AH->fSpec && strcmp(AH->fSpec, "") != 0)
 			AH->FH = fopen(AH->fSpec, PG_BINARY_R);
 		else
 			AH->FH = stdin;
 		if (!AH->FH)
-			die_horribly(AH, "%s: unable to open archive file %s", progname, AH->fSpec);
+			die_horribly(AH, modulename, "could not open archive file %s: %s\n", AH->fSpec, strerror(errno));
 
-		ctx->hasSeek = (fseek(AH->FH, 0, SEEK_CUR) == 0);
+		ctx->hasSeek = checkSeek(AH->FH);
 
 		ReadHead(AH);
 		ReadToc(AH);
@@ -219,12 +213,11 @@ _ArchiveEntry(ArchiveHandle *AH, TocEntry *te)
 
 	ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
 	if (te->dataDumper)
-		ctx->dataPos = -1;
+		ctx->dataState = K_OFFSET_POS_NOT_SET;
 	else
-		ctx->dataPos = 0;
-	ctx->dataLen = 0;
-	te->formatData = (void *) ctx;
+		ctx->dataState = K_OFFSET_NO_DATA;
 
+	te->formatData = (void *) ctx;
 }
 
 /*
@@ -241,8 +234,7 @@ _WriteExtraToc(ArchiveHandle *AH, TocEntry *te)
 {
 	lclTocEntry *ctx = (lclTocEntry *) te->formatData;
 
-	WriteInt(AH, ctx->dataPos);
-	WriteInt(AH, ctx->dataLen);
+	WriteOffset(AH, ctx->dataPos, ctx->dataState);
 }
 
 /*
@@ -256,6 +248,7 @@ _WriteExtraToc(ArchiveHandle *AH, TocEntry *te)
 static void
 _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 {
+	int	junk;
 	lclTocEntry *ctx = (lclTocEntry *) te->formatData;
 
 	if (ctx == NULL)
@@ -264,8 +257,14 @@ _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 		te->formatData = (void *) ctx;
 	}
 
-	ctx->dataPos = ReadInt(AH);
-	ctx->dataLen = ReadInt(AH);
+	ctx->dataState = ReadOffset(AH, &(ctx->dataPos) );
+
+	/*
+	 * Prior to V1.7 (pg7.3), we dumped the data size as an int
+	 * now we don't dump it at all.
+	 */
+	if (AH->version < K_VERS_1_7)
+		junk = ReadInt(AH);
 }
 
 /*
@@ -280,7 +279,8 @@ _PrintExtraToc(ArchiveHandle *AH, TocEntry *te)
 {
 	lclTocEntry *ctx = (lclTocEntry *) te->formatData;
 
-	ahprintf(AH, "-- Data Pos: %d (Length %d)\n", ctx->dataPos, ctx->dataLen);
+	ahprintf(AH, "-- Data Pos: " INT64_FORMAT "\n",
+			 (int64) ctx->dataPos);
 }
 
 /*
@@ -300,12 +300,12 @@ _StartData(ArchiveHandle *AH, TocEntry *te)
 	lclTocEntry *tctx = (lclTocEntry *) te->formatData;
 
 	tctx->dataPos = _getFilePos(AH, ctx);
+	tctx->dataState = K_OFFSET_POS_SET;
 
 	_WriteByte(AH, BLK_DATA);	/* Block type */
 	WriteInt(AH, te->id);		/* For sanity check */
 
 	_StartDataCompressor(AH, te);
-
 }
 
 /*
@@ -318,8 +318,8 @@ _StartData(ArchiveHandle *AH, TocEntry *te)
  * Mandatory.
  *
  */
-static int
-_WriteData(ArchiveHandle *AH, const void *data, int dLen)
+static size_t
+_WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	z_streamp	zp = ctx->zp;
@@ -329,7 +329,7 @@ _WriteData(ArchiveHandle *AH, const void *data, int dLen)
 
 	while (zp->avail_in != 0)
 	{
-		/* printf("Deflating %d bytes\n", dLen); */
+		/* printf("Deflating %lu bytes\n", (unsigned long) dLen); */
 		_DoDeflate(AH, ctx, 0);
 	}
 	return dLen;
@@ -345,12 +345,10 @@ _WriteData(ArchiveHandle *AH, const void *data, int dLen)
 static void
 _EndData(ArchiveHandle *AH, TocEntry *te)
 {
-	lclContext *ctx = (lclContext *) AH->formatData;
-	lclTocEntry *tctx = (lclTocEntry *) te->formatData;
+/*	lclContext *ctx = (lclContext *) AH->formatData; */
+/*	lclTocEntry *tctx = (lclTocEntry *) te->formatData; */
 
 	_EndDataCompressor(AH, te);
-
-	tctx->dataLen = _getFilePos(AH, ctx) - tctx->dataPos;
 }
 
 /*
@@ -370,10 +368,10 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 	lclTocEntry *tctx = (lclTocEntry *) te->formatData;
 
 	tctx->dataPos = _getFilePos(AH, ctx);
+	tctx->dataState = K_OFFSET_POS_SET;
 
 	_WriteByte(AH, BLK_BLOBS);	/* Block type */
 	WriteInt(AH, te->id);		/* For sanity check */
-
 }
 
 /*
@@ -387,7 +385,7 @@ static void
 _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 {
 	if (oid == 0)
-		die_horribly(AH, "%s: illegal OID for BLOB (%d)\n", progname, oid);
+		die_horribly(AH, modulename, "invalid OID for large object\n");
 
 	WriteInt(AH, oid);
 	_StartDataCompressor(AH, te);
@@ -430,12 +428,11 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 	int			blkType;
 	int			found = 0;
 
-	if (tctx->dataPos == 0)
+	if (tctx->dataState == K_OFFSET_NO_DATA)
 		return;
 
-	if (!ctx->hasSeek || tctx->dataPos < 0)
+	if (!ctx->hasSeek || tctx->dataState == K_OFFSET_POS_NOT_SET)
 	{
-
 		/* Skip over unnecessary blocks until we get the one we want. */
 
 		found = 0;
@@ -444,73 +441,60 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 
 		while (id != te->id)
 		{
-
 			if ((TocIDRequired(AH, id, ropt) & 2) != 0)
-				die_horribly(AH, "%s: Dumping a specific TOC data block out of order is not supported"
-							 " without on this input stream (fseek required)\n", progname);
+				die_horribly(AH, modulename,
+							 "Dumping a specific TOC data block out of order is not supported"
+				  " without id on this input stream (fseek required)\n");
 
 			switch (blkType)
 			{
-
 				case BLK_DATA:
-
 					_skipData(AH);
 					break;
 
 				case BLK_BLOBS:
-
 					_skipBlobs(AH);
 					break;
 
 				default:		/* Always have a default */
-
-					die_horribly(AH, "%s: unrecognized data block type while searching archive %d\n",
-								 progname, blkType);
+					die_horribly(AH, modulename,
+								 "unrecognized data block type (%d) while searching archive\n",
+								 blkType);
 					break;
 			}
-
 			_readBlockHeader(AH, &blkType, &id);
-
 		}
-
 	}
 	else
 	{
-
 		/* Grab it */
-
-		if (fseek(AH->FH, tctx->dataPos, SEEK_SET) != 0)
-			die_horribly(AH, "%s: error %d in file seek\n", progname, errno);
+		if (fseeko(AH->FH, tctx->dataPos, SEEK_SET) != 0)
+			die_horribly(AH, modulename, "error during file seek: %s\n", strerror(errno));
 
 		_readBlockHeader(AH, &blkType, &id);
-
 	}
 
 	/* Are we sane? */
 	if (id != te->id)
-		die_horribly(AH, "%s: Found unexpected block ID (%d) when reading data - expected %d\n",
-					 progname, id, te->id);
+		die_horribly(AH, modulename, "found unexpected block ID (%d) when reading data - expected %d\n",
+					 id, te->id);
 
 	switch (blkType)
 	{
-
 		case BLK_DATA:
-
 			_PrintData(AH);
 			break;
 
 		case BLK_BLOBS:
-
 			if (!AH->connection)
-				die_horribly(AH, "%s: BLOBs can not be loaded without a database connection\n", progname);
+				die_horribly(AH, modulename, "large objects cannot be loaded without a database connection\n");
 
 			_LoadBlobs(AH);
 			break;
 
 		default:				/* Always have a default */
-
-			die_horribly(AH, "%s: unrecognized data block type %d while restoring archive\n",
-						 progname, blkType);
+			die_horribly(AH, modulename, "unrecognized data block type %d while restoring archive\n",
+						 blkType);
 			break;
 	}
 
@@ -525,14 +509,13 @@ _PrintData(ArchiveHandle *AH)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	z_streamp	zp = ctx->zp;
-	int			blkLen;
+	size_t		blkLen;
 	char	   *in = ctx->zlibIn;
-	int			cnt;
+	size_t		cnt;
 
 #ifdef HAVE_LIBZ
 	int			res;
 	char	   *out = ctx->zlibOut;
-
 #endif
 
 #ifdef HAVE_LIBZ
@@ -546,9 +529,8 @@ _PrintData(ArchiveHandle *AH)
 		zp->opaque = Z_NULL;
 
 		if (inflateInit(zp) != Z_OK)
-			die_horribly(AH, "%s: could not initialize compression library - %s\n", progname, zp->msg);
+			die_horribly(AH, modulename, "could not initialize compression library: %s\n", zp->msg);
 	}
-
 #endif
 
 	blkLen = ReadInt(AH);
@@ -560,7 +542,7 @@ _PrintData(ArchiveHandle *AH)
 			ctx->zlibIn = NULL;
 			ctx->zlibIn = (char *) malloc(blkLen + 1);
 			if (!ctx->zlibIn)
-				die_horribly(AH, "%s: failed to allocate decompression buffer\n", progname);
+				die_horribly(AH, modulename, "out of memory\n");
 
 			ctx->inSize = blkLen + 1;
 			in = ctx->zlibIn;
@@ -568,7 +550,9 @@ _PrintData(ArchiveHandle *AH)
 
 		cnt = fread(in, 1, blkLen, AH->FH);
 		if (cnt != blkLen)
-			die_horribly(AH, "%s: could not read data block - expected %d, got %d\n", progname, blkLen, cnt);
+			die_horribly(AH, modulename,
+				   "could not read data block - expected %lu, got %lu\n",
+						 (unsigned long) blkLen, (unsigned long) cnt);
 
 		ctx->filePos += blkLen;
 
@@ -579,14 +563,13 @@ _PrintData(ArchiveHandle *AH)
 
 		if (AH->compression != 0)
 		{
-
 			while (zp->avail_in != 0)
 			{
 				zp->next_out = out;
 				zp->avail_out = zlibOutSize;
 				res = inflate(zp, 0);
 				if (res != Z_OK && res != Z_STREAM_END)
-					die_horribly(AH, "%s: unable to uncompress data - %s\n", progname, zp->msg);
+					die_horribly(AH, modulename, "unable to uncompress data: %s\n", zp->msg);
 
 				out[zlibOutSize - zp->avail_out] = '\0';
 				ahwrite(out, 1, zlibOutSize - zp->avail_out, AH);
@@ -602,9 +585,7 @@ _PrintData(ArchiveHandle *AH)
 #ifdef HAVE_LIBZ
 		}
 #endif
-
 		blkLen = ReadInt(AH);
-
 	}
 
 #ifdef HAVE_LIBZ
@@ -618,20 +599,21 @@ _PrintData(ArchiveHandle *AH)
 			zp->avail_out = zlibOutSize;
 			res = inflate(zp, 0);
 			if (res != Z_OK && res != Z_STREAM_END)
-				die_horribly(AH, "%s: unable to uncompress data - %s\n", progname, zp->msg);
+				die_horribly(AH, modulename, "unable to uncompress data: %s\n", zp->msg);
 
 			out[zlibOutSize - zp->avail_out] = '\0';
 			ahwrite(out, 1, zlibOutSize - zp->avail_out, AH);
 		}
+		if (inflateEnd(zp) != Z_OK)
+			die_horribly(AH, modulename, "could not close compression library: %s\n", zp->msg);
 	}
 #endif
-
 }
 
 static void
 _LoadBlobs(ArchiveHandle *AH)
 {
-	int			oid;
+	Oid			oid;
 
 	StartRestoreBlobs(AH);
 
@@ -645,7 +627,6 @@ _LoadBlobs(ArchiveHandle *AH)
 	}
 
 	EndRestoreBlobs(AH);
-
 }
 
 /*
@@ -657,7 +638,7 @@ _LoadBlobs(ArchiveHandle *AH)
 static void
 _skipBlobs(ArchiveHandle *AH)
 {
-	int			oid;
+	Oid			oid;
 
 	oid = ReadInt(AH);
 	while (oid != 0)
@@ -676,9 +657,9 @@ static void
 _skipData(ArchiveHandle *AH)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			blkLen;
+	size_t		blkLen;
 	char	   *in = ctx->zlibIn;
-	int			cnt;
+	size_t		cnt;
 
 	blkLen = ReadInt(AH);
 	while (blkLen != 0)
@@ -692,13 +673,14 @@ _skipData(ArchiveHandle *AH)
 		}
 		cnt = fread(in, 1, blkLen, AH->FH);
 		if (cnt != blkLen)
-			die_horribly(AH, "%s: could not read data block - expected %d, got %d\n", progname, blkLen, cnt);
+			die_horribly(AH, modulename,
+				   "could not read data block - expected %lu, got %lu\n",
+						 (unsigned long) blkLen, (unsigned long) cnt);
 
 		ctx->filePos += blkLen;
 
 		blkLen = ReadInt(AH);
 	}
-
 }
 
 /*
@@ -720,7 +702,7 @@ _WriteByte(ArchiveHandle *AH, const int i)
 	if (res != EOF)
 		ctx->filePos += 1;
 	else
-		die_horribly(AH, "%s: could not write byte./n", progname);
+		die_horribly(AH, modulename, "could not write byte: %s\n", strerror(errno));
 	return res;
 }
 
@@ -754,16 +736,18 @@ _ReadByte(ArchiveHandle *AH)
  * These routines are only used to read & write headers & TOC.
  *
  */
-static int
-_WriteBuf(ArchiveHandle *AH, const void *buf, int len)
+static size_t
+_WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = fwrite(buf, 1, len, AH->FH);
 
 	if (res != len)
-		die_horribly(AH, "%s: write error in _WriteBuf (%d != %d)\n", progname, res, len);
+		die_horribly(AH, modulename,
+					 "write error in _WriteBuf (%lu != %lu)\n",
+					 (unsigned long) res, (unsigned long) len);
 
 	ctx->filePos += res;
 	return res;
@@ -778,14 +762,15 @@ _WriteBuf(ArchiveHandle *AH, const void *buf, int len)
  * These routines are only used to read & write headers & TOC.
  *
  */
-static int
-_ReadBuf(ArchiveHandle *AH, void *buf, int len)
+static size_t
+_ReadBuf(ArchiveHandle *AH, void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = fread(buf, 1, len, AH->FH);
 	ctx->filePos += res;
+
 	return res;
 }
 
@@ -808,12 +793,12 @@ static void
 _CloseArchive(ArchiveHandle *AH)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			tpos;
+	off_t		tpos;
 
 	if (AH->mode == archModeWrite)
 	{
 		WriteHead(AH);
-		tpos = ftell(AH->FH);
+		tpos = ftello(AH->FH);
 		WriteToc(AH);
 		ctx->dataStart = _getFilePos(AH, ctx);
 		WriteDataChunks(AH);
@@ -826,13 +811,13 @@ _CloseArchive(ArchiveHandle *AH)
 		 */
 		if (ctx->hasSeek)
 		{
-			fseek(AH->FH, tpos, SEEK_SET);
+			fseeko(AH->FH, tpos, SEEK_SET);
 			WriteToc(AH);
 		}
 	}
 
 	if (fclose(AH->FH) != 0)
-		die_horribly(AH, "%s: could not close archive file\n", progname);
+		die_horribly(AH, modulename, "could not close archive file: %s\n", strerror(errno));
 
 	AH->FH = NULL;
 }
@@ -845,16 +830,23 @@ _CloseArchive(ArchiveHandle *AH)
 /*
  * Get the current position in the archive file.
  */
-static int
+static off_t
 _getFilePos(ArchiveHandle *AH, lclContext *ctx)
 {
-	int			pos;
+	off_t		pos;
 
 	if (ctx->hasSeek)
 	{
-		pos = ftell(AH->FH);
+		pos = ftello(AH->FH);
 		if (pos != ctx->filePos)
-			fprintf(stderr, "Warning: ftell mismatch with filePos\n");
+		{
+			write_msg(modulename, "WARNING: ftell mismatch with expected position -- ftell used\n");
+			/*
+			 * Prior to 1.7 (pg7.3) we relied on the internally maintained pointer.
+			 * Now we rely on off_t always.
+			 * pos = ctx->filePos;
+			 */
+		}
 	}
 	else
 		pos = ctx->filePos;
@@ -899,13 +891,12 @@ _StartDataCompressor(ArchiveHandle *AH, TocEntry *te)
 		zp->opaque = Z_NULL;
 
 		if (deflateInit(zp, AH->compression) != Z_OK)
-			die_horribly(AH, "%s: could not initialize compression library - %s\n", progname, zp->msg);
+			die_horribly(AH, modulename, "could not initialize compression library: %s\n", zp->msg);
 	}
 
 #else
 
 	AH->compression = 0;
-
 #endif
 
 	/* Just be paranoid - maybe End is called after Start, with no Write */
@@ -932,14 +923,13 @@ _DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
 	{
 		res = deflate(zp, flush);
 		if (res == Z_STREAM_ERROR)
-			die_horribly(AH, "%s: could not compress data - %s\n", progname, zp->msg);
+			die_horribly(AH, modulename, "could not compress data: %s\n", zp->msg);
 
 		if (((flush == Z_FINISH) && (zp->avail_out < zlibOutSize))
 			|| (zp->avail_out == 0)
 			|| (zp->avail_in != 0)
 			)
 		{
-
 			/*
 			 * Extra paranoia: avoid zero-length chunks since a zero
 			 * length chunk is the EOF marker. This should never happen
@@ -947,14 +937,13 @@ _DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
 			 */
 			if (zp->avail_out < zlibOutSize)
 			{
-
 				/*
-				 * printf("Wrote %d byte deflated chunk\n", zlibOutSize -
-				 * zp->avail_out);
+				 * printf("Wrote %lu byte deflated chunk\n", (unsigned
+				 * long) (zlibOutSize - zp->avail_out));
 				 */
 				WriteInt(AH, zlibOutSize - zp->avail_out);
 				if (fwrite(out, 1, zlibOutSize - zp->avail_out, AH->FH) != (zlibOutSize - zp->avail_out))
-					die_horribly(AH, "%s: could not write compressed chunk\n", progname);
+					die_horribly(AH, modulename, "could not write compressed chunk\n");
 				ctx->filePos += zlibOutSize - zp->avail_out;
 			}
 			zp->next_out = out;
@@ -968,7 +957,7 @@ _DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
 		{
 			WriteInt(AH, zp->avail_in);
 			if (fwrite(zp->next_in, 1, zp->avail_in, AH->FH) != zp->avail_in)
-				die_horribly(AH, "%s: could not write uncompressed chunk\n", progname);
+				die_horribly(AH, modulename, "could not write uncompressed chunk\n");
 			ctx->filePos += zp->avail_in;
 			zp->avail_in = 0;
 		}
@@ -979,8 +968,6 @@ _DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
 				res = Z_STREAM_END;
 #endif
 		}
-
-
 	}
 
 #ifdef HAVE_LIBZ
@@ -988,7 +975,6 @@ _DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
 #else
 	return 1;
 #endif
-
 }
 
 /*
@@ -1017,7 +1003,7 @@ _EndDataCompressor(ArchiveHandle *AH, TocEntry *te)
 		} while (res != Z_STREAM_END);
 
 		if (deflateEnd(zp) != Z_OK)
-			die_horribly(AH, "%s: error closing compression stream - %s\n", progname, zp->msg);
+			die_horribly(AH, modulename, "could not close compression stream: %s\n", zp->msg);
 	}
 #endif
 
