@@ -4,7 +4,7 @@
 #endif
 
 
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
 typedef unsigned char slock_t;
 
 #define HAS_TEST_AND_SET
@@ -14,6 +14,11 @@ typedef unsigned char slock_t;
 
 #define HAS_TEST_AND_SET
 
+#elif defined(__powerpc64__)
+typedef unsigned long slock_t;
+
+#define HAS_TEST_AND_SET
+
 #elif defined(__powerpc__)
 typedef unsigned int slock_t;
 
@@ -44,4 +49,9 @@ typedef unsigned int slock_t;
 
 #define HAS_TEST_AND_SET
 
+#elif defined(__mc68000__)
+typedef unsigned char slock_t;
+
+#define HAS_TEST_AND_SET
+
 #endif
